@@ -6,10 +6,20 @@ use warnings;
 use Carp 'croak';
 use constant PLACEHOLDER => "\x{b1}\x{a4}\x{221e}";
 
-our @CARP_NOT = qw(DBIx::DBO DBIx::DBO::Table DBIx::DBO::Query DBIx::DBO::Row);
+our @CARP_NOT = qw(DBIx::DBO DBIx::DBO::DBD DBIx::DBO::Table DBIx::DBO::Query DBIx::DBO::Row);
+*DBIx::DBO::CARP_NOT = \@CARP_NOT;
+*DBIx::DBO::Table::CARP_NOT = \@CARP_NOT;
+*DBIx::DBO::Query::CARP_NOT = \@CARP_NOT;
+*DBIx::DBO::Row::CARP_NOT = \@CARP_NOT;
+
 our $placeholder = PLACEHOLDER;
 $placeholder = qr/\Q$placeholder/;
 
+sub _init_dbo {
+    my($class, $me) = @_;
+    return $me;
+}
+
 sub _get_table_schema {
     my($class, $me, $schema, $table) = @_;
 
@@ -23,15 +33,16 @@ sub _get_table_schema {
         'TABLE,VIEW,GLOBAL TEMPORARY,LOCAL TEMPORARY,SYSTEM TABLE')->fetchall_arrayref;
     # Then if we found nothing, try any type
     $info = $me->rdbh->table_info(undef, $q_schema, $q_table)->fetchall_arrayref if $info and @$info == 0;
-    croak 'Invalid table: '.$class->_qi($me, $table) unless $info and @$info == 1 and $info->[0][2] eq $table;
+    croak 'Invalid table: '.$class->_qi($me, $schema, $table) unless $info and @$info == 1 and $info->[0][2] eq $table;
     return $info->[0][1];
 }
 
 sub _get_column_info {
     my($class, $me, $schema, $table) = @_;
 
-    my $cols = $me->rdbh->column_info(undef, $schema, $table, '%')->fetchall_arrayref({});
-    croak 'Invalid table: '.$class->_qi($me, $table) unless @$cols;
+    my $cols = $me->rdbh->column_info(undef, $schema, $table, '%');
+    $cols = $cols && $cols->fetchall_arrayref({}) || [];
+    croak 'Invalid table: '.$class->_qi($me, $schema, $table) unless @$cols;
 
     map { $_->{COLUMN_NAME} => $_->{ORDINAL_POSITION} } @$cols;
 }
@@ -96,7 +107,7 @@ sub _selectall_hashref {
 
 sub _qi {
     my($class, $me, @id) = @_;
-    return $me->dbh->quote_identifier(@id) if $me->config('QuoteIdentifier');
+    return $me->rdbh->quote_identifier(@id) if $me->config('QuoteIdentifier');
     # Strip off any null/undef elements (ie schema)
     shift(@id) while @id and not (defined $id[0] and length $id[0]);
     join '.', @id;
@@ -106,7 +117,7 @@ sub _sql {
     my($class, $me, $sql, @bind) = @_;
     my $dbg = $me->config('DebugSQL') or return;
 
-    require Carp::Heavy if $Carp::VERSION < 1.12;
+    require Carp::Heavy if eval "$Carp::VERSION < 1.12";
     my $loc = Carp::short_error_loc();
     my %i = Carp::caller_info($loc);
     my $trace;
@@ -208,9 +219,8 @@ sub _build_from {
     $h->{from} = $class->_build_table($me, ($me->tables)[0]);
     for (my $i = 1; $i < $me->tables; $i++) {
         $h->{from} .= $h->{Join}[$i].$class->_build_table($me, ($me->tables)[$i]);
-        if ($h->{Join_On}[$i]) {
-            $h->{from} .= ' ON '.join(' AND ', $class->_build_where_chunk($me, $h->{From_Bind}, 'OR', $h->{Join_On}[$i]));
-        }
+        $h->{from} .= ' ON '.join(' AND ', $class->_build_where_chunk($me, $h->{From_Bind}, 'OR', $h->{Join_On}[$i]))
+            if $h->{Join_On}[$i];
     }
     $h->{from};
 }
@@ -399,6 +409,8 @@ sub _build_quick_where {
                 if (ref $val eq 'SCALAR' and $$val =~ /^\s*(?:NOT\s+)NULL\s*$/is) {
                     ' IS ';
                 } elsif (ref $val eq 'ARRAY') {
+                    croak 'Invalid value argument, IN requires at least 1 value' unless @$val;
+                    $val = { FUNC => join(',', ('?') x @$val), VAL => $val };
                     ' IN ';
                 } elsif (defined $val) {
                     ' = ';
@@ -482,7 +494,31 @@ sub _set_config {
     return $old;
 }
 
-# Table
+
+# Query methods
+sub _rows {
+    my($class, $me) = @_;
+    $me->_sth and ($me->{sth}{Executed} or $me->run)
+        or croak $me->rdbh->errstr;
+    my $rows = $me->_sth->rows;
+    $me->{Row_Count} = $rows == -1 ? undef : $rows;
+}
+
+sub _calc_found_rows {
+    my($class, $me) = @_;
+    local $me->{build_data}{limit} = '';
+    $me->{Found_Rows} = $me->count_rows;
+}
+
+
+# Table methods
+sub _save_last_insert_id {
+    #my($class, $me, $sth) = @_;
+    # Should be provided in a DBD specific method
+    # It is called after insert and must return the autogenerated ID
+    #return $sth->{Database}->last_insert_id(undef, @$me{qw(Schema Name)}, undef);
+}
+
 sub _fast_bulk_insert {
     my($class, $me, $sql, $cols, %opt) = @_;
 
@@ -538,31 +574,6 @@ sub _safe_bulk_insert {
 *_bulk_insert = \&_safe_bulk_insert;
 
 
-# Query methods
-sub _rows {
-    my($class, $me) = @_;
-    $me->_sth and ($me->{sth}{Executed} or $me->run)
-        or croak $me->rdbh->errstr;
-    my $rows = $me->_sth->rows;
-    $me->{Row_Count} = $rows == -1 ? undef : $rows;
-}
-
-sub _calc_found_rows {
-    my($class, $me) = @_;
-    local $me->{build_data}{limit} = '';
-    $me->{Found_Rows} = $me->count_rows;
-}
-
-
-# Table methods
-sub _save_last_insert_id {
-    #my($class, $me, $sth) = @_;
-    # Should be provided in a DBD specific method
-    # It is called after insert and must return the autogenerated ID
-    #return $sth->{Database}->last_insert_id(undef, @$me{qw(Schema Name)}, undef);
-}
-
-
 # Row methods
 sub _reset_row_on_update {
     my($class, $me, @update) = @_;
